@@ -27,21 +27,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-/*
- *  $Id: defs.h,v 1.43 2003/05/21 10:40:27 pavlin Exp $
- */
-/*
- * Part of this program has been derived from mrouted.
- * The mrouted program is covered by the license in the accompanying file
- * named "LICENSE.mrouted".
- *
- * The mrouted program is COPYRIGHT 1989 by The Board of Trustees of
- * Leland Stanford Junior University.
- *
- */
-/*
- *  $Id: defs.h,v 1.43 2003/05/21 10:40:27 pavlin Exp $
- */
 /*
  * Part of this program has been derived from mrouted.
  * The mrouted program is covered by the license in the accompanying file
@@ -51,6 +36,8 @@
  * Leland Stanford Junior University.
  *
  */
+#ifndef __PIMD_DEFS_H__
+#define __PIMD_DEFS_H__
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -70,7 +57,7 @@
 #endif /* SYSV || bsdi || SunOS 4.x */
 #include <time.h>
 #include <sys/time.h>
-#include <linux/if.h>
+#include <net/if.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
@@ -83,17 +70,26 @@
 #include <net/route.h>
 #undef rtentry
 #endif /* bsdi or __FreeBSD_version >= 220000 */
-#ifdef Linux
-#if 1
-#include <netinet/mroute.h>
-#endif /* 0 */
-/* TODO: XXX: needed for RedHat-6.1, maybe delete for other versions? */
-#if 0
+#ifdef __linux__
+#define _LINUX_IN_H             /* For Linux <= 2.6.25 */
+#include <linux/types.h>
 #include <linux/mroute.h>
-#endif /* 0 */
 #else
 #include <netinet/ip_mroute.h>
-#endif /* Linux */
+#endif /* __linux__ */
+#if defined(HAVE_STRLCPY)
+#include <string.h>
+#endif
+#if defined(HAVE_STRTONUM)
+#include <stdlib.h>
+#endif
+#if defined(HAVE_PIDFILE)
+#if defined(OpenBSD) || defined(NetBSD)
+#include <util.h>
+#else
+#include <libutil.h>
+#endif
+#endif
 #include <strings.h>
 #ifdef RSRR
 #include <sys/un.h>
@@ -107,7 +103,7 @@ typedef u_char  u_int8;
 #if (BSD >= 199103)
 #include <machine/endian.h>
 #else
-#ifdef linux
+#ifdef __linux__
 #include <endian.h>
 #else
 #define LITTLE_ENDIAN	1234	/* least-significant byte first (vax, pc) */
@@ -134,16 +130,8 @@ typedef u_char  u_int8;
 #endif /* BSD */
 #endif /* BYTE_ORDER */
 
-#ifndef __P
-#ifdef __STDC__
-#define __P(x)  x
-#else
-#define __P(x)  ()
-#endif
-#endif
-
-typedef void (*cfunc_t) __P((void *));
-typedef void (*ihfunc_t) __P((int, fd_set *));
+typedef void (*cfunc_t) (void *);
+typedef void (*ihfunc_t) (int, fd_set *);
 
 #include "dvmrp.h"     /* Added for further compatibility and convenience */
 #include "pimd.h"
@@ -170,13 +158,13 @@ typedef void (*ihfunc_t) __P((int, fd_set *));
  * Various definitions to make it working for different platforms
  */
 /* The old style sockaddr definition doesn't have sa_len */
-#if (defined(BSD) && (BSD >= 199006)) /* sa_len was added with 4.3-Reno */ 
+#if defined(_AIX) || (defined(BSD) && (BSD >= 199006)) /* sa_len was added with 4.3-Reno */
 #define HAVE_SA_LEN
 #endif
 
 /* Versions of Solaris older than 2.6 don't have routing sockets. */
 /* XXX TODO: check FreeBSD version and add all other platforms */
-#if defined (Linux) || (defined(SunOS) && SunOS >=56) || defined (__FreeBSD__) || defined (IRIX) || defined (__bsdi__) || defined(NetBSD) || defined(OpenBSD)
+#if defined(__linux__) || (defined(SunOS) && SunOS >=56) || defined (__FreeBSD__) || defined (IRIX) || defined (__bsdi__) || defined(NetBSD) || defined(OpenBSD)
 #define HAVE_ROUTING_SOCKETS	1
 #endif
 
@@ -195,15 +183,9 @@ typedef void (*ihfunc_t) __P((int, fd_set *));
 #define MFC_UPDATE_FORCE	0x2
 
 #define EQUAL(s1, s2)		(strcmp((s1), (s2)) == 0)
+#define ARRAY_LEN(a)            (sizeof((a)) / sizeof((a)[0]))
 
-/* obnoxious gcc gives an extraneous warning about this constant... */
-#if defined(__STDC__) || defined(__GNUC__)
-#define JAN_1970        2208988800UL    /* 1970 - 1900 in seconds */
-#else
-#define JAN_1970        2208988800L     /* 1970 - 1900 in seconds */
-#define const           /**/
-#endif
-
+#define JAN_1970                2208988800UL    /* 1970 - 1900 in seconds */
 
 #define MINTTL			1  /* min TTL in the packets send locally */
 
@@ -264,7 +246,6 @@ typedef void (*ihfunc_t) __P((int, fd_set *));
 
 #ifdef SYSV
 #define bcopy(a, b, c)		memcpy((b), (a), (c))
-#define bzero(s, n)		memset((s), 0, (n))
 #define setlinebuf(s)		setvbuf((s), (NULL), (_IOLBF), 0)
 #define RANDOM()                lrand48()
 #else
@@ -296,13 +277,10 @@ extern u_int32		allpimrouters_group;
 extern build_jp_message_t *build_jp_message_pool;
 extern int		build_jp_message_pool_counter;
 
-#ifdef RSRR
-extern int		rsrr_socket;
-#endif /* RSRR */
-
 extern u_long		virtual_time;
-extern char		configfilename[];
-/* extern int		pid; Removed due problems with ia64, Debian Bug #287915*/
+extern char	       *configfilename;
+extern int              haveterminal;
+extern char            *__progname;
 
 extern struct cand_rp_adv_message_ {
     u_int8    *buffer;
@@ -363,12 +341,13 @@ extern int		udp_socket;
 
 extern int		vifs_down;
 
-extern char		s1[];
-extern char		s2[];
-extern char		s3[];
-extern char		s4[];
+#define MAX_INET_BUF_LEN 19
+extern char		s1[MAX_INET_BUF_LEN];
+extern char		s2[MAX_INET_BUF_LEN];
+extern char		s3[MAX_INET_BUF_LEN];
+extern char		s4[MAX_INET_BUF_LEN];
 
-#if !((defined(BSD) && (BSD >= 199103)) || (defined(Linux)))
+#if !((defined(BSD) && (BSD >= 199103)) || (defined(__linux__)))
 extern int		errno;
 extern int		sys_nerr;
 extern char *		sys_errlist[];
@@ -377,7 +356,7 @@ extern char *		sys_errlist[];
 
 #ifndef IGMP_MEMBERSHIP_QUERY
 #define IGMP_MEMBERSHIP_QUERY		IGMP_HOST_MEMBERSHIP_QUERY
-#if !(defined(NetBSD) || defined(OpenBSD))
+#if !(defined(NetBSD) || defined(OpenBSD) || defined(__FreeBSD__))
 #define IGMP_V1_MEMBERSHIP_REPORT	IGMP_HOST_MEMBERSHIP_REPORT
 #define IGMP_V2_MEMBERSHIP_REPORT	IGMP_HOST_NEW_MEMBERSHIP_REPORT
 #else
@@ -387,7 +366,7 @@ extern char *		sys_errlist[];
 #define IGMP_V2_LEAVE_GROUP		IGMP_HOST_LEAVE_MESSAGE
 #endif
 
-#if defined(NetBSD) || defined(OpenBSD)
+#if defined(NetBSD) || defined(OpenBSD) || defined(__FreeBSD__)
 #define IGMP_MTRACE_RESP		IGMP_MTRACE_REPLY
 #define IGMP_MTRACE			IGMP_MTRACE_QUERY
 #endif
@@ -444,246 +423,167 @@ extern char *		sys_errlist[];
  */
 
 /* callout.c */
-extern void	callout_init		__P((void));
-extern void	free_all_callouts	__P((void));
-extern void	age_callout_queue	__P((int));
-extern int	timer_nextTimer		__P((void));
-extern int	timer_setTimer		__P((int, cfunc_t, void *));
-extern void	timer_clearTimer	__P((int));
-extern int	timer_leftTimer		__P((int));
+extern void	callout_init		(void);
+extern void	free_all_callouts	(void);
+extern void	age_callout_queue	(int);
+extern int	timer_nextTimer		(void);
+extern int	timer_setTimer		(int, cfunc_t, void *);
+extern void	timer_clearTimer	(int);
+extern int	timer_leftTimer		(int);
 
 /* config.c */
-extern void	config_vifs_from_kernel	__P(());
-extern void	config_vifs_from_file	__P(());
+extern void	config_vifs_from_kernel	(void);
+extern void	config_vifs_from_file	(void);
 
 /* debug.c */
-extern char	*packet_kind		__P((u_int proto, u_int type,
-					     u_int code));
-extern int	debug_kind		__P((u_int proto, u_int type,
-					     u_int code));
-extern void	pimd_log			__P((int, int, const char *, ...));
-extern int	log_level		__P((u_int proto, u_int type,
-					     u_int code));
-extern void	dump			__P((int i));
-extern void	fdump			__P((int i));
-extern void	cdump			__P((int i));
-extern void	dump_vifs		__P((FILE *fp));
-extern void	dump_pim_mrt		__P((FILE *fp));
-extern int	dump_rp_set		__P((FILE *fp));
+extern char	*packet_kind		(u_int proto, u_int type, u_int code);
+extern int	debug_kind		(u_int proto, u_int type, u_int code);
+extern void	logit			(int, int, const char *, ...);
+extern int	log_level		(u_int proto, u_int type, u_int code);
+extern void	dump			(int i);
+extern void	fdump			(int i);
+extern void	cdump			(int i);
+extern void	dump_vifs		(FILE *fp);
+extern void	dump_pim_mrt		(FILE *fp);
+extern int	dump_rp_set		(FILE *fp);
 
 /* dvmrp_proto.c */
-extern void	dvmrp_accept_probe	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen,
-					     u_int32 level));
-extern void	dvmrp_accept_report	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen,
-					     u_int32 level));
-extern void	dvmrp_accept_info_request __P((u_int32 src, u_int32 dst,
-					       u_char *p, int datalen));
-extern void	dvmrp_accept_info_reply	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen));
-extern void	dvmrp_accept_neighbors	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen,
-					     u_int32 level));
-extern void	dvmrp_accept_neighbors2	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen,
-					     u_int32 level));
-extern void	dvmrp_accept_prune	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen));
-extern void	dvmrp_accept_graft	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen));
-extern void	dvmrp_accept_g_ack	__P((u_int32 src, u_int32 dst,
-					     u_char *p, int datalen));
+extern void	dvmrp_accept_probe	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
+extern void	dvmrp_accept_report	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
+extern void	dvmrp_accept_info_request (u_int32 src, u_int32 dst, u_char *p, int datalen);
+extern void	dvmrp_accept_info_reply	(u_int32 src, u_int32 dst, u_char *p, int datalen);
+extern void	dvmrp_accept_neighbors	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
+extern void	dvmrp_accept_neighbors2	(u_int32 src, u_int32 dst, u_char *p, int datalen, u_int32 level);
+extern void	dvmrp_accept_prune	(u_int32 src, u_int32 dst, u_char *p, int datalen);
+extern void	dvmrp_accept_graft	(u_int32 src, u_int32 dst, u_char *p, int datalen);
+extern void	dvmrp_accept_g_ack	(u_int32 src, u_int32 dst, u_char *p, int datalen);
 
 /* igmp.c */
-extern void	init_igmp		__P(());
-extern void	send_igmp		__P((char *buf, u_int32 src,
-					     u_int32 dst, int type, int code,
-					     u_int32 group, int datalen));
+extern void	init_igmp		(void);
+extern void	send_igmp		(char *buf, u_int32 src, u_int32 dst, int type, int code, u_int32 group, int datalen);
 
 /* igmp_proto.c */
-extern void	query_groups		__P((struct uvif *v));
-extern void	accept_membership_query	__P((u_int32 src, u_int32 dst,
-					     u_int32 group, int tmo));
-extern void	accept_group_report	__P((u_int32 src, u_int32 dst,
-					     u_int32 group, int r_type));
-extern void	accept_leave_message	__P((u_int32 src, u_int32 dst,
-					     u_int32 group));
+extern void	query_groups		(struct uvif *v);
+extern void	accept_membership_query	(u_int32 src, u_int32 dst, u_int32 group, int tmo);
+extern void	accept_group_report	(u_int32 src, u_int32 dst, u_int32 group, int r_type);
+extern void	accept_leave_message	(u_int32 src, u_int32 dst, u_int32 group);
 
 /* inet.c */
-extern int	inet_cksum		__P((u_int16 *addr, u_int len));
-extern int	inet_valid_host		__P((u_int32 naddr));
-extern int	inet_valid_mask		__P((u_int32 mask));
-extern int	inet_valid_subnet	__P((u_int32 nsubnet, u_int32 nmask));
-extern char	*inet_fmt		__P((u_int32, char *s));
-extern char	*netname		__P((u_int32 addr, u_int32 mask));
-extern u_int32	inet_parse		__P((char *s, int n));
+extern int	inet_cksum		(u_int16 *addr, u_int len);
+extern int	inet_valid_host		(u_int32 naddr);
+extern int	inet_valid_mask		(u_int32 mask);
+extern int	inet_valid_subnet	(u_int32 nsubnet, u_int32 nmask);
+extern char	*inet_fmt		(u_int32 addr, char *s, size_t len);
+extern char	*netname		(u_int32 addr, u_int32 mask);
+extern u_int32	inet_parse		(char *s, int n);
 
 /* kern.c */
-extern void	k_set_sndbuf		__P((int socket, int bufsize,
-					     int minsize));
-extern void	k_set_rcvbuf		__P((int socket, int bufsize,
-					     int minsize));
-extern void	k_hdr_include		__P((int socket, int bool));
-extern void	k_set_ttl		__P((int socket, int t));
-extern void	k_set_loop		__P((int socket, int l));
-extern void	k_set_if		__P((int socket, u_int32 ifa));
-extern void	k_join			__P((int socket, u_int32 grp,
-					     struct uvif *v));
-extern void	k_leave			__P((int socket, u_int32 grp,
-					     struct uvif *v));
-extern void	k_init_pim		__P(());
-extern void	k_stop_pim		__P(());
-extern int	k_del_mfc		__P((int socket, u_int32 source,
-					     u_int32 group));
-extern int	k_chg_mfc		__P((int socket, u_int32 source,
-					     u_int32 group, vifi_t iif,
-					     vifbitmap_t oifs,
-					     u_int32 rp_addr));
-extern void	k_add_vif		__P((int socket, vifi_t vifi,
-					     struct uvif *v));
-extern void	k_del_vif		__P((int socket, vifi_t vifi));
-extern int	k_get_vif_count		__P((vifi_t vifi,
-					     struct vif_count *retval));
-extern int	k_get_sg_cnt		__P((int socket, u_int32 source,
-					     u_int32 group,
-					     struct sg_count *retval));
+extern void	k_set_sndbuf		(int socket, int bufsize, int minsize);
+extern void	k_set_rcvbuf		(int socket, int bufsize, int minsize);
+extern void	k_hdr_include		(int socket, int bool);
+extern void	k_set_ttl		(int socket, int t);
+extern void	k_set_loop		(int socket, int l);
+extern void	k_set_if		(int socket, u_int32 ifa);
+extern void	k_join			(int socket, u_int32 grp, struct uvif *v);
+extern void	k_leave			(int socket, u_int32 grp, struct uvif *v);
+extern void	k_init_pim		(int socket);
+extern void	k_stop_pim		(int socket);
+extern int	k_del_mfc		(int socket, u_int32 source, u_int32 group);
+extern int	k_chg_mfc		(int socket, u_int32 source, u_int32 group, vifi_t iif, vifbitmap_t oifs,
+                                         u_int32 rp_addr);
+extern void	k_add_vif		(int socket, vifi_t vifi, struct uvif *v);
+extern void	k_del_vif		(int socket, vifi_t vifi);
+extern int	k_get_vif_count		(vifi_t vifi, struct vif_count *retval);
+extern int	k_get_sg_cnt		(int socket, u_int32 source, u_int32 group, struct sg_count *retval);
 
 /* main.c */
-extern int	register_input_handler	__P((int fd, ihfunc_t func));
+extern int	register_input_handler	(int fd, ihfunc_t func);
 
 /* mrt.c */
-extern void	init_pim_mrt		__P(());
-extern mrtentry_t *find_route		__P((u_int32 source, u_int32 group,
-					     u_int16 flags, char create));
-extern grpentry_t *find_group		__P((u_int32 group));
-extern srcentry_t *find_source		__P((u_int32 source));
-extern void	delete_mrtentry		__P((mrtentry_t *mrtentry_ptr));
-extern void	delete_srcentry		__P((srcentry_t *srcentry_ptr));
-extern void	delete_grpentry		__P((grpentry_t *grpentry_ptr));
-extern void	delete_mrtentry_all_kernel_cache __P((mrtentry_t *mrtentry_ptr));
-extern void	delete_single_kernel_cache __P((mrtentry_t *mrtentry_ptr,
-						kernel_cache_t *kernel_cache_ptr));
-extern void	delete_single_kernel_cache_addr __P((mrtentry_t *mrtentry_ptr,
-						     u_int32 source,
-						     u_int32 group));
-extern void	add_kernel_cache	__P((mrtentry_t *mrtentry_ptr,
-					     u_int32 source, u_int32 group,
-					     u_int16 flags));
+extern void	init_pim_mrt		(void);
+extern mrtentry_t *find_route		(u_int32 source, u_int32 group, u_int16 flags, char create);
+extern grpentry_t *find_group		(u_int32 group);
+extern srcentry_t *find_source		(u_int32 source);
+extern void	delete_mrtentry		(mrtentry_t *mrtentry_ptr);
+extern void	delete_srcentry		(srcentry_t *srcentry_ptr);
+extern void	delete_grpentry		(grpentry_t *grpentry_ptr);
+extern void	delete_mrtentry_all_kernel_cache (mrtentry_t *mrtentry_ptr);
+extern void	delete_single_kernel_cache (mrtentry_t *mrtentry_ptr, kernel_cache_t *kernel_cache_ptr);
+extern void	delete_single_kernel_cache_addr (mrtentry_t *mrtentry_ptr, u_int32 source, u_int32 group);
+extern void	add_kernel_cache	(mrtentry_t *mrtentry_ptr, u_int32 source, u_int32 group, u_int16 flags);
 /* pim.c */
-extern void	init_pim		__P(());
-extern void	send_pim		__P((char *buf, u_int32 src,
-					     u_int32 dst, int type,
-					     int datalen));
-extern void	send_pim_unicast	__P((char *buf, u_int32 src,
-					     u_int32 dst, int type,
-					     int datalen));
+extern void	init_pim		(void);
+extern void	send_pim		(char *buf, u_int32 src, u_int32 dst, int type, int datalen);
+extern void	send_pim_unicast	(char *buf, u_int32 src, u_int32 dst, int type, int datalen);
 
 /* pim_proto.c */
-extern int	receive_pim_hello	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, size_t datalen));
-extern int	send_pim_hello		__P((struct uvif *v,
-					     u_int16 holdtime));
-extern void	delete_pim_nbr		__P((pim_nbr_entry_t *nbr_delete));
-extern int	receive_pim_register	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, size_t datalen));
-extern int	send_pim_null_register	__P((mrtentry_t *r));
-extern int	receive_pim_register_stop __P((u_int32 src, u_int32 dst,
-					       char *pim_message,
-					       size_t datalen));
-extern int	send_pim_register	__P((char *pkt));
-extern int	receive_pim_join_prune	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, int datalen));
-extern int	join_or_prune		__P((mrtentry_t *mrtentry_ptr,
-					     pim_nbr_entry_t *upstream_router));
-extern int	receive_pim_assert	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, int datalen));
-extern int	send_pim_assert		__P((u_int32 source, u_int32 group,
-					     vifi_t vifi,
-					     mrtentry_t *mrtentry_ptr));
-extern int	send_periodic_pim_join_prune __P((vifi_t vifi,
-						  pim_nbr_entry_t *pim_nbr,
-						  u_int16 holdtime));
-extern int	add_jp_entry		__P((pim_nbr_entry_t *pim_nbr,
-					     u_int16 holdtime, u_int32 group,
-					     u_int8 grp_msklen, u_int32 source,
-					     u_int8 src_msklen,
-					     u_int16 addr_flags,
-					     u_int8 join_prune));
-extern void	pack_and_send_jp_message __P((pim_nbr_entry_t *pim_nbr));
-extern int	receive_pim_cand_rp_adv	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, int datalen));
-extern int	receive_pim_bootstrap	__P((u_int32 src, u_int32 dst,
-					     char *pim_message, int datalen));
-extern int	send_pim_cand_rp_adv	__P(());
-extern void	send_pim_bootstrap	__P(());
+extern int	receive_pim_hello	(u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
+extern int	send_pim_hello		(struct uvif *v, u_int16 holdtime);
+extern void	delete_pim_nbr		(pim_nbr_entry_t *nbr_delete);
+extern int	receive_pim_register	(u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
+extern int	send_pim_null_register	(mrtentry_t *r);
+extern int	receive_pim_register_stop (u_int32 src, u_int32 dst, char *pim_message, size_t datalen);
+extern int	send_pim_register	(char *pkt);
+extern int	receive_pim_join_prune	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	join_or_prune		(mrtentry_t *mrtentry_ptr, pim_nbr_entry_t *upstream_router);
+extern int	receive_pim_assert	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	send_pim_assert		(u_int32 source, u_int32 group, vifi_t vifi, mrtentry_t *mrtentry_ptr);
+extern int	send_periodic_pim_join_prune (vifi_t vifi, pim_nbr_entry_t *pim_nbr, u_int16 holdtime);
+extern int	add_jp_entry		(pim_nbr_entry_t *pim_nbr, u_int16 holdtime, u_int32 group, u_int8 grp_msklen,
+                                         u_int32 source, u_int8 src_msklen,  u_int16 addr_flags, u_int8 join_prune);
+extern void	pack_and_send_jp_message (pim_nbr_entry_t *pim_nbr);
+extern int	receive_pim_cand_rp_adv	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	receive_pim_bootstrap	(u_int32 src, u_int32 dst, char *pim_message, int datalen);
+extern int	send_pim_cand_rp_adv	(void);
+extern void	send_pim_bootstrap	(void);
 
 /* route.c */
-extern int	set_incoming		__P((srcentry_t *srcentry_ptr,
-					     int srctype));
-extern vifi_t	get_iif			__P((u_int32 source));
-extern pim_nbr_entry_t *find_pim_nbr	__P((u_int32 source));
-extern int	add_sg_oif		__P((mrtentry_t *mrtentry_ptr,
-					     vifi_t vifi,
-					     u_int16 holdtime,
-					     int update_holdtime));
-extern void	add_leaf		__P((vifi_t vifi, u_int32 source,
-					     u_int32 group));
-extern void	delete_leaf		__P((vifi_t vifi, u_int32 source,
-					     u_int32 group));
-extern int	change_interfaces	__P((mrtentry_t *mrtentry_ptr,
-					     vifi_t new_iif,
-					     vifbitmap_t new_joined_oifs_,
-					     vifbitmap_t new_pruned_oifs,
-					     vifbitmap_t new_leaves_,
-					     vifbitmap_t new_asserted_oifs,
-					     u_int16 flags));
-extern void	calc_oifs		__P((mrtentry_t *mrtentry_ptr,
-					     vifbitmap_t *oifs_ptr));
-extern void	process_kernel_call	__P(());
-extern int	delete_vif_from_mrt	__P((vifi_t vifi));
-extern mrtentry_t *switch_shortest_path	__P((u_int32 source, u_int32 group));
-
+extern int	set_incoming		(srcentry_t *srcentry_ptr, int srctype);
+extern vifi_t	get_iif			(u_int32 source);
+extern pim_nbr_entry_t *find_pim_nbr	(u_int32 source);
+extern int	add_sg_oif		(mrtentry_t *mrtentry_ptr, vifi_t vifi, u_int16 holdtime, int update_holdtime);
+extern void	add_leaf		(vifi_t vifi, u_int32 source, u_int32 group);
+extern void	delete_leaf		(vifi_t vifi, u_int32 source, u_int32 group);
+extern int	change_interfaces	(mrtentry_t *mrtentry_ptr,  vifi_t new_iif,
+                                         vifbitmap_t new_joined_oifs_, vifbitmap_t new_pruned_oifs,
+                                         vifbitmap_t new_leaves_, vifbitmap_t new_asserted_oifs, u_int16 flags);
+extern void	calc_oifs		(mrtentry_t *mrtentry_ptr, vifbitmap_t *oifs_ptr);
+extern void	process_kernel_call	(void);
+extern int	delete_vif_from_mrt	(vifi_t vifi);
+extern mrtentry_t *switch_shortest_path	(u_int32 source, u_int32 group);
 
 /* routesock.c */
-extern int	k_req_incoming		__P((u_int32 source,
-					     struct rpfctl *rpfp));
+extern int	k_req_incoming		(u_int32 source, struct rpfctl *rpfp);
 #ifdef HAVE_ROUTING_SOCKETS
-extern int	init_routesock		__P(());
+extern int	init_routesock		(void);
 extern int	routing_socket;
 #endif /* HAVE_ROUTING_SOCKETS */
 
 /* rp.c */
-extern void	init_rp_and_bsr		__P(());
-extern u_int16	bootstrap_initial_delay __P(());
-extern rp_grp_entry_t *add_rp_grp_entry __P((cand_rp_t  **used_cand_rp_list,
-					     grp_mask_t **used_grp_mask_list,
-					     u_int32 rp_addr,
-					     u_int8  rp_priority,
-					     u_int16 rp_holdtime,
-					     u_int32 group_addr,
-					     u_int32 group_mask,
-					     u_int32 bsr_hash_mask,
-					     u_int16 fragment_tag));
-extern void	delete_rp_grp_entry	__P((cand_rp_t  **used_cand_rp_list,
-					     grp_mask_t **used_grp_mask_list,
-					     rp_grp_entry_t *rp_grp_entry_delete));
-extern void	delete_grp_mask		__P((cand_rp_t  **used_cand_rp_list,
-					     grp_mask_t **used_grp_mask_list,
-					     u_int32 group_addr,
-					     u_int32 group_mask));
-extern void	delete_rp		__P((cand_rp_t  **used_cand_rp_list,
-					     grp_mask_t **used_grp_mask_list,
-					     u_int32 rp_addr));
-extern void	delete_rp_list		__P((cand_rp_t  **used_cand_rp_list,
-					     grp_mask_t **used_grp_mask_list));
-extern rpentry_t *rp_match		__P((u_int32 group));
-extern rp_grp_entry_t *rp_grp_match	__P((u_int32 group));
-extern rpentry_t *rp_find		__P((u_int32 rp_address));
-extern int	remap_grpentry		__P((grpentry_t *grpentry_ptr));
-extern int	create_pim_bootstrap_message __P((char *send_buff));
-extern int	check_mrtentry_rp	__P((mrtentry_t *mrtentry_ptr,
-					     u_int32 rp_addr));
+extern void	init_rp_and_bsr		(void);
+extern u_int16	bootstrap_initial_delay (void);
+extern rp_grp_entry_t *add_rp_grp_entry (cand_rp_t  **used_cand_rp_list,
+                                         grp_mask_t **used_grp_mask_list,
+                                         u_int32 rp_addr,
+                                         u_int8  rp_priority,
+                                         u_int16 rp_holdtime,
+                                         u_int32 group_addr,
+                                         u_int32 group_mask,
+                                         u_int32 bsr_hash_mask,
+                                         u_int16 fragment_tag);
+extern void	delete_rp_grp_entry	(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
+                                         rp_grp_entry_t *rp_grp_entry_delete);
+extern void	delete_grp_mask		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
+                                         u_int32 group_addr, u_int32 group_mask);
+extern void	delete_rp		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list,
+                                         u_int32 rp_addr);
+extern void	delete_rp_list		(cand_rp_t  **used_cand_rp_list, grp_mask_t **used_grp_mask_list);
+extern rpentry_t *rp_match		(u_int32 group);
+extern rp_grp_entry_t *rp_grp_match	(u_int32 group);
+extern rpentry_t *rp_find		(u_int32 rp_address);
+extern int	remap_grpentry		(grpentry_t *grpentry_ptr);
+extern int	create_pim_bootstrap_message (char *send_buff);
+extern int	check_mrtentry_rp	(mrtentry_t *mrtentry_ptr, u_int32 rp_addr);
 
 #ifdef RSRR
 #ifdef PIM
@@ -693,38 +593,36 @@ extern int	check_mrtentry_rp	__P((mrtentry_t *mrtentry_ptr,
 #define RSRR_NOTIFICATION_FALSE		FALSE
 
 /* rsrr.c */
-extern void	rsrr_init		__P((void));
-extern void	rsrr_clean		__P((void));
-extern void	rsrr_cache_send		__P((struct gtable *, int));
-extern void	rsrr_cache_clean	__P((struct gtable *));
-extern void	rsrr_cache_bring_up	__P((struct gtable *));
+extern void	rsrr_init		(void);
+extern void	rsrr_clean		(void);
+extern void	rsrr_cache_send		(struct gtable *, int);
+extern void	rsrr_cache_clean	(struct gtable *);
+extern void	rsrr_cache_bring_up	(struct gtable *);
 #endif /* RSRR */
 
 /* timer.c */
-extern void	init_timers		__P(());
-extern void	age_vifs		__P(());
-extern void	age_routes		__P(());
-extern void	age_misc		__P(());
-extern int	unicast_routing_changes	__P((srcentry_t *src_ent));
-extern int	clean_srclist		__P(());
+extern void	init_timers		(void);
+extern void	age_vifs		(void);
+extern void	age_routes		(void);
+extern void	age_misc		(void);
+extern int	unicast_routing_changes	(srcentry_t *src_ent);
+extern int	clean_srclist		(void);
 
 /* trace.c */
 /* u_int is promoted u_char */
-extern void	accept_mtrace		__P((u_int32 src, u_int32 dst,
-					     u_int32 group, char *data,
-					     u_int no, int datalen));
-extern void	accept_neighbor_request	__P((u_int32 src, u_int32 dst));
-extern void	accept_neighbor_request2 __P((u_int32 src, u_int32 dst));
+extern void	accept_mtrace		(u_int32 src, u_int32 dst, u_int32 group, char *data, u_int no, int datalen);
+extern void	accept_neighbor_request	(u_int32 src, u_int32 dst);
+extern void	accept_neighbor_request2 (u_int32 src, u_int32 dst);
 
 /* vif.c */
-extern void	init_vifs		__P(());
-extern void	zero_vif		__P((struct uvif *, int));
-extern void	stop_all_vifs		__P(());
-extern void	check_vif_state		__P(());
-extern vifi_t	local_address		__P((u_int32 src));
-extern vifi_t	find_vif_direct		__P((u_int32 src));
-extern vifi_t	find_vif_direct_local	__P((u_int32 src));
-extern u_int32	max_local_address	__P((void));
+extern void	init_vifs		(void);
+extern void	zero_vif		(struct uvif *, int);
+extern void	stop_all_vifs		(void);
+extern void	check_vif_state		(void);
+extern vifi_t	local_address		(u_int32 src);
+extern vifi_t	find_vif_direct		(u_int32 src);
+extern vifi_t	find_vif_direct_local	(u_int32 src);
+extern u_int32	max_local_address	(void);
 
 struct rp_hold {
 	struct rp_hold *next;
@@ -732,3 +630,12 @@ struct rp_hold {
 	u_int32	group;
 	u_int32	mask;
 };
+
+#ifndef HAVE_STRLCPY
+size_t strlcpy(char *dst, const char *src, size_t siz);
+#endif
+#ifndef HAVE_PIDFILE
+int pidfile(const char *basename);
+#endif
+
+#endif /* __PIMD_DEFS_H__ */
