@@ -42,26 +42,21 @@
 
 
 #include "defs.h"
-
-struct rp_hold *g_rp_hold=NULL;
-
-
+#include <err.h>
+#include <getopt.h>
+#include <paths.h>
 #ifdef SNMP
 #include "snmp.h"
 #endif
 
-char configfilename[256] = _PATH_PIMD_CONF;
 char versionstring[100];
-int pid = 0;
 int disable_all_by_default = 0;
-
-static char pidfilename[]  = _PATH_PIMD_PID;
-/* TODO: not used
-static char genidfilename[] = _PATH_PIMD_GENID;
-*/
-
 int haveterminal = 1;
-char *progname;
+struct rp_hold *g_rp_hold = NULL;
+
+char *configfilename = _PATH_PIMD_CONF;
+
+extern char todaysversion[];
 
 static int sighandled = 0;
 #define GOT_SIGINT      0x01
@@ -84,16 +79,16 @@ static struct ihandler {
 static int nhandlers = 0;
 
 static struct debugname {
-    char *name;
-    int	 level;
-    u_int nchars;
+    char	*name;
+    uint32_t	 level;
+    size_t	 nchars;
 } debugnames[] = {
     {   "dvmrp_detail",	    DEBUG_DVMRP_DETAIL,   5	    },
     {   "dvmrp_prunes",	    DEBUG_DVMRP_PRUNE,    8	    },
     {   "dvmrp_pruning",    DEBUG_DVMRP_PRUNE,    8	    },
-    {   "dvmrp_mrt",        DEBUG_DVMRP_ROUTE,    7	    },
     {   "dvmrp_routes",	    DEBUG_DVMRP_ROUTE,    7	    },
     {   "dvmrp_routing",    DEBUG_DVMRP_ROUTE,    7	    },
+    {   "dvmrp_mrt",        DEBUG_DVMRP_ROUTE,    7	    },
     {   "dvmrp_neighbors",  DEBUG_DVMRP_PEER,     7	    },
     {   "dvmrp_peers",	    DEBUG_DVMRP_PEER,     8	    },
     {   "dvmrp_hello",      DEBUG_DVMRP_PEER,     7	    },
@@ -110,8 +105,8 @@ static struct debugname {
     {   "traceroute",       DEBUG_TRACE,          2	    },
     {   "timeout",	    DEBUG_TIMEOUT,        2	    },
     {   "callout",	    DEBUG_TIMEOUT,        3	    },
-    {   "pkt",	            DEBUG_PKT,  	  2	    },
     {   "packets",	    DEBUG_PKT,  	  2	    },
+    {   "pkt",	            DEBUG_PKT,  	  2	    },
     {   "interfaces",       DEBUG_IF,   	  2	    },
     {   "vif",	            DEBUG_IF,   	  1	    },
     {   "kernel",           DEBUG_KERN,           2	    },
@@ -123,6 +118,7 @@ static struct debugname {
     {   "pim_detail",       DEBUG_PIM_DETAIL,     5	    },
     {   "pim_hello",        DEBUG_PIM_HELLO,      5	    },
     {   "pim_neighbors",    DEBUG_PIM_HELLO,      5	    },
+    {   "pim_peers",        DEBUG_PIM_HELLO,      5	    },
     {   "pim_register",     DEBUG_PIM_REGISTER,   5	    },
     {   "registers",        DEBUG_PIM_REGISTER,   2	    },
     {   "pim_join_prune",   DEBUG_PIM_JOIN_PRUNE, 5	    },
@@ -147,9 +143,10 @@ static struct debugname {
     {   "routes",	    DEBUG_MRT,            1	    },
     {   "routing",	    DEBUG_MRT,            1	    },
     {   "mrt",  	    DEBUG_MRT,            1	    },
+    {   "neighbors",        DEBUG_NEIGHBORS,      1	    },
     {   "routers",          DEBUG_NEIGHBORS,      6	    },
     {   "mrouters",         DEBUG_NEIGHBORS,      7	    },
-    {   "neighbors",        DEBUG_NEIGHBORS,      1	    },
+    {   "peers",            DEBUG_NEIGHBORS,      1	    },
     {   "timers",           DEBUG_TIMER,          1	    },
     {   "asserts",          DEBUG_ASSERT,         1	    },
     {   "all",              DEBUG_ALL,            2         },
@@ -159,20 +156,13 @@ static struct debugname {
 /*
  * Forward declarations.
  */
-static void handler __P((int));
-static void timer __P((void *));
-static void cleanup __P((void));
-static void restart __P((int));
-static void resetlogging __P((void *));
+static void handler     (int);
+static void timer        (void *);
+static void cleanup      (void);
+static void restart      (int);
+static void resetlogging (void *);
 
-
-/* To shut up gcc -Wstrict-prototypes */
-int main __P((int argc, char **argv));
-
-int
-register_input_handler(fd, func)
-    int fd;
-ihfunc_t func;
+int register_input_handler(int fd, ihfunc_t func)
 {
     if (nhandlers >= NHANDLERS)
 	return -1;
@@ -183,137 +173,254 @@ ihfunc_t func;
     return 0;
 }
 
-int
-main(argc, argv)
-    int argc;
-    char *argv[];
-{	
-    int dummy, dummysigalrm;
+static void do_randomize(void)
+{
+#define rol32(data,shift) ((data) >> (shift)) | ((data) << (32 - (shift)))
+   int fd;
+   unsigned int seed;
+
+   /* Setup a fallback seed based on quasi random. */
+#ifdef SYSV
+   seed = time(NULL);
+#else
+   seed = time(NULL) ^ gethostid();
+#endif
+   seed = rol32(seed, seed);
+
+   fd = open("/dev/urandom", O_RDONLY);
+   if (fd >= 0) {
+       if (-1 == read(fd, &seed, sizeof(seed)))
+	   warn("Failed reading entropy from /dev/urandom");
+       close(fd);
+  }
+
+#ifdef SYSV
+   srand48(seed);
+#else
+   srandom(seed);
+#endif
+}
+
+/* Figure out the PID of a running daemon. */
+static pid_t daemon_pid(void)
+{
+    int result;
+    char *path = NULL;
     FILE *fp;
+    pid_t pid = -1;
+
+    result = asprintf(&path, "%s%s.pid", _PATH_VARRUN, __progname);
+    if (result == -1 || path == NULL)
+	return -1;
+
+    fp = fopen(path, "r");
+    if (!fp) {
+	free(path);
+	return -1;
+    }
+
+    result = fscanf(fp, "%d", &pid);
+    fclose(fp);
+    free(path);
+
+    return pid;
+}
+
+/* Send signal to running daemon and the show resulting file. */
+static void killshow(int signo, char *file)
+{
+    pid_t pid = daemon_pid();
+    char buf[100];
+
+    if (pid > 0) {
+	if (file)
+	    remove(file);
+	kill(pid, signo);
+	if (file) {
+	    int result;
+
+	    usleep(200);
+	    snprintf(buf, sizeof(buf), "cat %s", file);
+	    result = system(buf);
+	}
+    }
+}
+
+static int usage(void)
+{
+    size_t i, j, k;
+    struct debugname *d;
+
+    fprintf(stderr, "Usage: %s [-fhN] [-c FILE] [-d [LEVEL][,LEVEL...]]\n\n", __progname);
+    fputs("  -c, --config=FILE    Configuration file to use, default /etc/pimd.conf\n", stderr);
+    fputs("  -d, --debug[=LEVEL]  Debug level, see below for valid levels\n", stderr);
+    fputs("  -f, --foreground     Run in foreground, do not detach from calling terminal\n", stderr);
+    fputs("  -h, --help           Show this help text\n", stderr);
+//    fputs("  -i, --show-cache      Show internal cache tables\n", stderr);
+    fputs("  -l, --reload-config  Tell a running pimd to reload its configuration\n", stderr);
+    fputs("  -N, --disable-vifs   Disable all virtual interfaces (phyint) by default\n", stderr);
+//    fputs("  -p,--show-debug      Show debug dump, only if debug is enabled\n", stderr);
+    fputs("  -q, --quit-daemon    Send SIGTERM to a running pimd\n", stderr);
+    fputs("  -r, --show-routes    Show state of VIFs and multicast routing tables\n", stderr);
+    fprintf(stderr, "  -v, --version        Show %s version\n", __progname);
+    fputs("\n", stderr);
+
+    j = 0xffffffff;
+    k = 0;
+    fputs("Valid debug levels:\n  ", stderr);
+    for (i = 0, d = debugnames; i < ARRAY_LEN(debugnames); i++, d++) {
+	if ((j & d->level) == d->level) {
+	    if (k++) {
+		if (!(k % 5))
+		    fputs("\n  ", stderr);
+		else
+		    fputs(", ", stderr);
+	    }
+	    fputs(d->name, stderr);
+	    j &= ~d->level;
+	}
+    }
+    fputc('\n', stderr);
+
+    return 1;
+}
+
+int main(int argc, char *argv[])
+{	
+    int dummy, dummysigalrm, foreground = 0;
     struct timeval tv, difftime, curtime, lasttime, *timeout;
     fd_set rfds, readers;
-    int nfds, n, i, secs;
-    extern char todaysversion[];
+    int nfds, n, i, secs, ch;
     struct sigaction sa;
-    struct debugname *d;
-    char c;
-    int tmpd;
     time_t boottime;
-
-    setlinebuf(stderr);
-
-    progname = strrchr(argv[0], '/');
-    if (progname)
-	progname++;
-    else
-	progname = argv[0];
-	
-    if (geteuid() != 0) {
-	fprintf(stderr, "%s: must be root\n", progname);
-	exit(1);
-    }
-    pid = getpid();
+    struct option long_options[] = {
+	{"config", 1, 0, 'c'},
+	{"debug", 2, 0, 'd'},
+	{"foreground", 0, 0, 'f'},
+	{"disable-vifs", 0, 0, 'N'},
+	{"help", 0, 0, 'h'},
+	{"version", 0, 0, 'v'},
+	{"quit-daemon", 0, 0, 'q'},
+	{"reload-config", 0, 0, 'l'},
+	{"show-routes", 0, 0, 'r'},
+//	{"show-cache", 0, 0, 'i'},
+//	{"show-debug", 0, 0, 'p'},
+	{0, 0, 0, 0}
+    };
     
-    argv++;
-    argc--;
-    while (argc > 0 && *argv[0] == '-') {
-	if (strcmp(*argv, "-d") == 0) {
-	    if (argc > 1 && *(argv + 1)[0] != '-') { 
-		char *p,*q;
-		u_int i, len;
-		struct debugname *d;
-		
-		argv++;
-		argc--;
-		debug = 0;
-		p = *argv; q = NULL;
-		while (p) {
-		    q = strchr(p, ',');
-		    if (q)
-			*q++ = '\0';
-		    len = strlen(p);
-		    for (i = 0, d = debugnames;
-			 i < sizeof(debugnames) / sizeof(debugnames[0]);
-			 i++, d++)
-			if (len >= d->nchars && strncmp(d->name, p, len) == 0)
-			    break;
-		    if (i == sizeof(debugnames) / sizeof(debugnames[0])) {
-			int j = 0xffffffff;
-			int k = 0;
-			fprintf(stderr, "Valid debug levels: ");
-			for (i = 0, d = debugnames;
-			     i < sizeof(debugnames) / sizeof(debugnames[0]);
-			     i++, d++) {
-			    if ((j & d->level) == d->level) {
-				if (k++)
-				    putc(',', stderr);
-				fputs(d->name, stderr);
-				j &= ~d->level;
-			    }
-			}
-			putc('\n', stderr);
-			goto usage;
+    snprintf(versionstring, sizeof (versionstring), "pimd version %s", todaysversion);
+
+    while ((ch = getopt_long (argc, argv, "c:d::fhlNP::vqr", long_options, NULL)) != EOF) {
+	switch (ch) {
+	    case 'c':
+		configfilename = optarg;
+		break;
+
+	    case 'd':
+		if (!optarg) {
+		    debug = DEBUG_DEFAULT;
+		} else {
+		    char *p,*q;
+		    size_t i, len;
+		    struct debugname *d;
+
+		    debug = 0;
+		    p = optarg; q = NULL;
+		    while (p) {
+			q = strchr(p, ',');
+			if (q)
+			    *q++ = '\0';
+			len = strlen(p);
+			for (i = 0, d = debugnames; i < ARRAY_LEN(debugnames); i++, d++)
+			    if (len >= d->nchars && strncmp(d->name, p, len) == 0)
+				break;
+
+			if (i == ARRAY_LEN(debugnames))
+			    return usage();
+
+			debug |= d->level;
+			p = q;
 		    }
-		    debug |= d->level;
-		    p = q;
 		}
-	    }
-	    else
-		debug = DEBUG_DEFAULT;
-	}
-	else if (strcmp(*argv, "-c") == 0) {
-	    if (argc > 1) {
-		argv++; argc--;
-		strcpy(configfilename, *argv);
-	    }
-	    else
-		goto usage;
-/* TODO: not implemented */
-#ifdef SNMP
-	}
-	else if (strcmp(*argv, "-P") == 0) {
-	    if (argc > 1 && isdigit(*(argv + 1)[0])) {
-		argv++, argc--;
-		dest_port = atoi(*argv);
-	    }
-	    else
-		dest_port = DEFAULT_PORT;
-#endif
-	}
-	else if (strcmp(*argv, "-N") == 0) {
+		break;
+
+	    case 'f':
+		foreground = 1;
+		break;
+
+	    case 'h':
+		return usage();
+
+	    case 'l':
+		killshow(SIGHUP, NULL);
+		return 0;
+
+	    case 'N':
 		disable_all_by_default = 1;
+		break;
+
+	    case 'P':
+#ifdef SNMP
+		if (!optarg)
+		    dest_port = DEFAULT_PORT;
+		else {
+		    dest_port = strtonum(optarg, 1, 65535, &errstr);
+		    if (errstr) {
+			warnx("destination port %s", errstr);
+			dest_port = DEFAULT_PORT;
+		    }
+		}
+#else
+		warnx("SNMP support missing, please feel free to submit a patch.");
+#endif
+		break;
+
+	    case 'v':
+		printf("%s\n", versionstring);
+		return 0;
+
+	    case 'q':
+		killshow(SIGTERM, NULL);
+		return 0;
+
+	    case 'r':
+		killshow(SIGUSR1, _PATH_PIMD_DUMP);
+		return 0;
+#if 0 /* XXX: TODO */
+	    case 'i':
+		killshow(SIGUSR2, _PATH_PIMD_CACHE);
+		return 0;
+
+	    case 'p':
+		killshow(SIGQUIT, NULL);
+		return 0;
+#endif
+	    default:
+		return usage();
 	}
-	else
-	    goto usage;
-	argv++; argc--;
     }
 
+    argc -= optind;
+    argv += optind;
+
     if (argc > 0) {
-    usage:
-	tmpd = 0xffffffff;
-	fprintf(stderr, "usage: pimd [-c configfile] [-N] [-d [debug_level][,debug_level]]\n");
-	
-	fprintf(stderr, "debug levels: ");
-    c = '(';
-    for (d = debugnames; d < debugnames +
-	     sizeof(debugnames) / sizeof(debugnames[0]); d++) {
-	if ((tmpd & d->level) == d->level) {
-	    tmpd &= ~d->level;
-	    fprintf(stderr, "%c%s", c, d->name);
-	    c = ',';
-	}
+	return usage();
     }
-    fprintf(stderr, ")\n");
-    fprintf(stderr, "-N: causes all interfaces found in kernel to be disabled by default\n");
-    exit(1);
-    }	
-    
+
+    if (geteuid() != 0) {
+	fprintf(stderr, "%s: must be root\n", __progname);
+	exit(1);
+    }
+    setlinebuf(stderr);
+
     if (debug != 0) {
-	tmpd = debug;
+	struct debugname *d;
+	char c;
+	int tmpd = debug;
+
 	fprintf(stderr, "debug level 0x%lx ", debug);
 	c = '(';
-	for (d = debugnames; d < debugnames +
-		 sizeof(debugnames) / sizeof(debugnames[0]); d++) {
+	for (d = debugnames; d < debugnames + ARRAY_LEN(debugnames); d++) {
 	    if ((tmpd & d->level) == d->level) {
 		tmpd &= ~d->level;
 		fprintf(stderr, "%c%s", c, d->name);
@@ -329,19 +436,10 @@ main(argc, argv)
 #else
     (void)openlog("pimd", LOG_PID);
 #endif /* LOG_DAEMON */
-    sprintf(versionstring, "pimd version %s", todaysversion);
-    
-    pimd_log(LOG_DEBUG, 0, "%s starting", versionstring);
-    
-/* TODO: XXX: use a combination of time and hostid to initialize the random
- * generator.
- */
-#ifdef SYSV
-    srand48(time(NULL));
-#else
-    srandom(gethostid());
-#endif
 
+    logit(LOG_DEBUG, 0, "%s starting", versionstring);
+
+    do_randomize();
     time(&boottime);
     
     /* Start up the log rate-limiter */
@@ -396,7 +494,7 @@ main(argc, argv)
     /* schedule first timer interrupt */
     timer_setTimer(TIMER_INTERVAL, timer, NULL);
     
-    if (debug == 0) {
+    if (!debug && !foreground) {
 	/* Detach from the terminal */
 	haveterminal = 0;
 	if (fork())
@@ -407,7 +505,7 @@ main(argc, argv)
 	(void)open("/", 0);
 	(void)dup2(0, 1);
 	(void)dup2(0, 2);
-#if defined(SYSV) || defined(__USE_SVID)
+#ifdef SYSV
 	(void)setpgrp();
 #else 
 #ifdef TIOCNOTTY
@@ -422,14 +520,11 @@ main(argc, argv)
 #endif /* TIOCNOTTY */
 #endif /* SYSV */
     } /* End of child process code */
-    pid = getpid();	/* XXX: if a child fork, the pid may change */
     
-    fp = fopen(pidfilename, "w");
-    if (fp != NULL) {
-	fprintf(fp, "%d\n", (int)getpid());
-	(void) fclose(fp);
+    if (pidfile (NULL)) {
+	warn("Cannot create pidfile");
     }
-    
+
     /*
      * Main receive loop.
      */
@@ -438,7 +533,7 @@ main(argc, argv)
     difftime.tv_usec = 0;
     gettimeofday(&curtime, NULL);
     lasttime = curtime;
-    for(;;) {
+    while (1) {
 	bcopy((char *)&readers, (char *)&rfds, sizeof(rfds));
 	secs = timer_nextTimer();
 	if (secs == -1)
@@ -501,7 +596,7 @@ main(argc, argv)
 	}
 	if ((n = select(nfds, &rfds, NULL, NULL, timeout)) < 0) {
 	    if (errno != EINTR) /* SIGALRM is expected */
-		pimd_log(LOG_WARNING, errno, "select failed");
+		logit(LOG_WARNING, errno, "select failed");
 	    continue;
 	}
 	if (n > 0) {
@@ -555,7 +650,7 @@ main(argc, argv)
 	} while (difftime.tv_sec > 0);
     } /* Main loop */
 
-    pimd_log(LOG_NOTICE, 0, "%s exiting", versionstring);
+    logit(LOG_NOTICE, 0, "%s exiting", versionstring);
     cleanup();
     exit(0);
 }
@@ -584,9 +679,7 @@ u_long virtual_time = 0;
  * aging interfaces, quering neighbors and members, etc... The granularity
  * is equal to TIMER_INTERVAL.
  */
-static void 
-timer(i)
-    void *i __attribute__((unused));
+static void timer(void *i __attribute__((unused)))
 {
     age_vifs();	        /* Timeout neighbors and groups         */
     age_routes();  	/* Timeout routing entries              */
@@ -600,8 +693,7 @@ timer(i)
  * Performs all necessary functions to quit gracefully
  */
 /* TODO: implement all necessary stuff */
-static void
-cleanup()
+static void cleanup(void)
 {
     vifi_t vifi;
     struct uvif *v;
@@ -630,9 +722,7 @@ cleanup()
  * Signal handler.  Take note of the fact that the signal arrived
  * so that the main loop can take care of it.
  */
-static void
-handler(sig)
-    int sig;
+static void handler(int sig)
 {
     switch (sig) {
     case SIGALRM:
@@ -661,15 +751,13 @@ handler(sig)
 /*
  * Restart the daemon
  */
-static void
-restart(i)
-    int i __attribute__((unused));
+static void restart(int i __attribute__((unused)))
 {
 #ifdef SNMP
     int s;
 #endif /* SNMP */
     
-    pimd_log(LOG_NOTICE, 0, "%s % restart", versionstring);
+    logit(LOG_NOTICE, 0, "%s % restart", versionstring);
     
     /*
      * reset all the entries
@@ -710,18 +798,13 @@ restart(i)
 	exit(s);
 #endif /* SNMP */
     init_vifs();
-#ifdef RSRR
-    rsrr_init();
-#endif /* RSRR */
 
     /* schedule timer interrupts */
     timer_setTimer(TIMER_INTERVAL, timer, NULL);
 }
 
 
-static void
-resetlogging(arg)
-    void *arg;
+static void resetlogging(void *arg)
 {
     int nxttime = 60;
     void *narg = NULL;
@@ -737,3 +820,12 @@ resetlogging(arg)
     
     timer_setTimer(nxttime, resetlogging, narg);
 }
+
+/**
+ * Local Variables:
+ *  version-control: t
+ *  indent-tabs-mode: t
+ *  c-file-style: "ellemtel"
+ *  c-basic-offset: 4
+ * End:
+ */
