@@ -1,27 +1,31 @@
 /*
- * Copyright (c) 1993, 1998 by the University of Southern California
+ * Copyright (c) 1993, 1998-2001.
+ * The University of Southern California/Information Sciences Institute.
  * All rights reserved.
  *
- * Permission to use, copy, modify, and distribute this software and its
- * documentation in source and binary forms for lawful purposes
- * and without fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both the copyright notice and
- * this permission notice appear in supporting documentation. and that
- * any documentation, advertising materials, and other materials related
- * to such distribution and use acknowledge that the software was
- * developed by the University of Southern California, Information
- * Sciences Institute.  The name of the University may not be used to
- * endorse or promote products derived from this software without
- * specific prior written permission.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. Neither the name of the project nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
- * the suitability of this software for any purpose.  THIS SOFTWARE IS
- * PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
- * INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- *
- * Other copyrights might apply to parts of this software and are so
- * noted when applicable.
+ * THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 /* RSRR code written by Daniel Zappala, USC Information Sciences Institute,
@@ -34,159 +38,131 @@
 
 #include "defs.h"
 #include <sys/param.h>
-#if (defined(BSD) && (BSD >= 199103))
-#include <stddef.h>
+#ifdef HAVE_SA_LEN
+#include <stddef.h>		/* for offsetof */
 #endif
 
-
-/*
- * Exported variables.
- */
-int rsrr_socket;		/* interface to reservation protocol */
-
 /* 
- * Global RSRR variables.
+ * Local RSRR variables.
  */
-char *rsrr_recv_buf;    	/* RSRR receive buffer */
-char *rsrr_send_buf;    	/* RSRR send buffer */
-
-struct sockaddr_un client_addr;
-int client_length = sizeof(client_addr);
+static int rsrr_socket;		/* interface to reservation protocol */
+static char *rsrr_recv_buf;    	/* RSRR receive buffer */
+static char *rsrr_send_buf;    	/* RSRR send buffer */
 
+static struct sockaddr_un client_addr;
+static socklen_t client_length = sizeof(client_addr);
 
 /*
- * Local functions definition
+ * Procedure definitions needed internally.
  */
-static void	rsrr_accept    __P((int recvlen));
-static void	rsrr_accept_iq __P((void));
-static int	rsrr_accept_rq __P((struct rsrr_rq *route_query, u_int8 flags,
-				    struct gtable *gt_notify));
-static void	rsrr_read      __P((int, fd_set *));
-static int	rsrr_send      __P((int sendlen));
-static void	rsrr_cache     __P((struct gtable *gt,
-					struct rsrr_rq *route_query));
+static void	rsrr_accept    (size_t recvlen);
+static void	rsrr_accept_iq (void);
+static int	rsrr_accept_rq (struct rsrr_rq *route_query, u_int8 flags, struct gtable *gt_notify);
+static void	rsrr_read      (int, fd_set *);
+static int	rsrr_send      (int sendlen);
+static void	rsrr_cache     (struct gtable *gt, struct rsrr_rq *route_query);
 
 /* Initialize RSRR socket */
-void
-rsrr_init()
+void rsrr_init(void)
 {
     int servlen;
     struct sockaddr_un serv_addr;
 
-    rsrr_recv_buf = malloc(RSRR_MAX_LEN);
-    rsrr_send_buf = malloc(RSRR_MAX_LEN);
+    rsrr_recv_buf = (char *)malloc(RSRR_MAX_LEN);
+    rsrr_send_buf = (char *)malloc(RSRR_MAX_LEN);
+    if (!rsrr_recv_buf || !rsrr_send_buf)
+	logit(LOG_ERR, 0, "Ran out of memory in rsrr_init()");
 
     if ((rsrr_socket = socket(AF_UNIX, SOCK_DGRAM, 0)) < 0)
-	pimd_log(LOG_ERR, errno, "Can't create RSRR socket");
+	logit(LOG_ERR, errno, "Cannot create RSRR socket");
 
     unlink(RSRR_SERV_PATH);
-    bzero((char *) &serv_addr, sizeof(serv_addr));
+    memset(&serv_addr, 0, sizeof(serv_addr));
     serv_addr.sun_family = AF_UNIX;
-    strcpy(serv_addr.sun_path, RSRR_SERV_PATH);
-#if (defined(BSD) && (BSD >= 199103))
-    servlen = offsetof(struct sockaddr_un, sun_path) +
-		strlen(serv_addr.sun_path);
+    strlcpy(serv_addr.sun_path, RSRR_SERV_PATH, sizeof(serv_addr.sun_path));
+#ifdef HAVE_SA_LEN
+    servlen = offsetof(struct sockaddr_un, sun_path) + strlen(serv_addr.sun_path);
     serv_addr.sun_len = servlen;
 #else
     servlen = sizeof(serv_addr.sun_family) + strlen(serv_addr.sun_path);
 #endif
  
     if (bind(rsrr_socket, (struct sockaddr *) &serv_addr, servlen) < 0)
-	pimd_log(LOG_ERR, errno, "Can't bind RSRR socket");
+	logit(LOG_ERR, errno, "Cannot bind RSRR socket");
 
     if (register_input_handler(rsrr_socket, rsrr_read) < 0)
-	pimd_log(LOG_ERR, 0, "Couldn't register RSRR as an input handler");
+	logit(LOG_ERR, 0, "Could not register RSRR as an input handler");
 }
 
 /* Read a message from the RSRR socket */
-static void
-rsrr_read(f, rfd)
-	int f;
-	fd_set *rfd;
+static void rsrr_read(int fd, fd_set *rfd __attribute__ ((unused)))
 {
-    register int rsrr_recvlen;
+    ssize_t len;
     
-    bzero((char *) &client_addr, sizeof(client_addr));
-    rsrr_recvlen = recvfrom(rsrr_socket, rsrr_recv_buf, sizeof(rsrr_recv_buf),
-			    0, (struct sockaddr *)&client_addr,
-			    &client_length);
-    if (rsrr_recvlen < 0) {	
-	if (errno != EINTR)
-	    pimd_log(LOG_ERR, errno, "RSRR recvfrom");
+    memset(&client_addr, 0, sizeof(client_addr));
+    while ((len = recvfrom(fd, rsrr_recv_buf, sizeof(rsrr_recv_buf),
+			   0, (struct sockaddr *)&client_addr, &client_length)) < 0) {
+	if (errno == EINTR)
+	    continue;		/* Received signal, retry syscall. */
+
+	logit(LOG_ERR, errno, "Failed recvfrom() in rsrr_read()");
 	return;
     }
-    rsrr_accept(rsrr_recvlen);
+
+    rsrr_accept(len);
 }
 
-/*
- * Accept a message from the reservation protocol and take
+/* Accept a message from the reservation protocol and take
  * appropriate action.
  */
-static void
-rsrr_accept(recvlen)
-    int recvlen;
+static void rsrr_accept(size_t recvlen)
 {
-    struct rsrr_header *rsrr;
+    struct rsrr_header *rsrr = (struct rsrr_header *)rsrr_recv_buf;
     struct rsrr_rq *route_query;
     
     if (recvlen < RSRR_HEADER_LEN) {
-	pimd_log(LOG_WARNING, 0,
-	    "Received RSRR packet of %d bytes, which is less than min size",
-	    recvlen);
+	logit(LOG_WARNING, 0, "Received RSRR packet of %d bytes, which is less than MIN size %d.",
+	      recvlen, RSRR_HEADER_LEN);
 	return;
     }
     
-    rsrr = (struct rsrr_header *) rsrr_recv_buf;
-    
-    if (rsrr->version > RSRR_MAX_VERSION) {
-	pimd_log(LOG_WARNING, 0,
-	    "Received RSRR packet version %d, which I don't understand",
-	    rsrr->version);
+    if (rsrr->version > RSRR_MAX_VERSION || rsrr->version != 1) {
+	logit(LOG_WARNING, 0, "Received RSRR packet version %d, which I don't understand",
+	      rsrr->version);
 	return;
     }
-    
-    switch (rsrr->version) {
-      case 1:
-	switch (rsrr->type) {
-	  case RSRR_INITIAL_QUERY:
+
+    switch (rsrr->type) {
+	case RSRR_INITIAL_QUERY:
 	    /* Send Initial Reply to client */
-	    IF_DEBUG(DEBUG_RSRR)
-		pimd_log(LOG_DEBUG, 0, "Received Initial Query\n");
+	    IF_DEBUG(DEBUG_RSRR) {
+		logit(LOG_DEBUG, 0, "Received Initial Query\n");
+	    }
 	    rsrr_accept_iq();
 	    break;
-	  case RSRR_ROUTE_QUERY:
+
+	case RSRR_ROUTE_QUERY:
 	    /* Check size */
 	    if (recvlen < RSRR_RQ_LEN) {
-		pimd_log(LOG_WARNING, 0,
-		    "Received Route Query of %d bytes, which is too small",
-		    recvlen);
+		logit(LOG_WARNING, 0, "Received Route Query of %d bytes, which is too small", recvlen);
 		break;
 	    }
 	    /* Get the query */
-	    route_query =
-		(struct rsrr_rq *) (rsrr_recv_buf + RSRR_HEADER_LEN);
-	    IF_DEBUG(DEBUG_RSRR)
-		pimd_log(LOG_DEBUG, 0,
-		    "Received Route Query for src %s grp %s notification %d",
-		    inet_fmt(route_query->source_addr, s1),
-		    inet_fmt(route_query->dest_addr, s2),
-		    BIT_TST(rsrr->flags, RSRR_NOTIFICATION_BIT));
+	    route_query = (struct rsrr_rq *) (rsrr_recv_buf + RSRR_HEADER_LEN);
+	    IF_DEBUG(DEBUG_RSRR) {
+		logit(LOG_DEBUG, 0,
+		      "Received Route Query for src %s grp %s notification %d",
+		      inet_fmt(route_query->source_addr, s1, sizeof(s1)),
+		      inet_fmt(route_query->dest_addr, s2, sizeof(s2)),
+		      BIT_TST(rsrr->flags,RSRR_NOTIFICATION_BIT));
+	    }
 	    /* Send Route Reply to client */
-	    rsrr_accept_rq(route_query, rsrr->flags, (struct gtable *)NULL);
+	    rsrr_accept_rq(route_query, rsrr->flags, NULL);
 	    break;
-	  default:
-	    pimd_log(LOG_WARNING, 0,
-		"Received RSRR packet type %d, which I don't handle",
-		rsrr->type);
+
+	default:
+	    logit(LOG_WARNING, 0, "Received RSRR packet type %d, which I don't handle", rsrr->type);
 	    break;
-	}
-	break;
-	
-      default:
-	pimd_log(LOG_WARNING, 0,
-	    "Received RSRR packet version %d, which I don't understand",
-	    rsrr->version);
-	break;
     }
 }
 
@@ -198,10 +174,9 @@ rsrr_accept(recvlen)
  * available anymore from the internet-draft ftp sites). Probably has to
  * be included in RSRRv2.
  */
-static void
-rsrr_accept_iq()
+static void rsrr_accept_iq(void)
 {
-    struct rsrr_header *rsrr;
+    struct rsrr_header *rsrr = (struct rsrr_header *)rsrr_send_buf;
     struct rsrr_vif *vif_list;
     struct uvif *v;
     vifi_t vifi;
@@ -211,27 +186,26 @@ rsrr_accept_iq()
      * but we should check anyway.
      */
     if (numvifs > RSRR_MAX_VIFS) {
-	pimd_log(LOG_WARNING, 0,
-	    "Can't send RSRR Route Reply because %d is too many vifs %d",
+	logit(LOG_WARNING, 0,
+	    "Cannot send RSRR Route Reply because %d is too many vifs %d",
 	    numvifs);
 	return;
     }
     
     /* Set up message */
-    rsrr = (struct rsrr_header *) rsrr_send_buf;
     rsrr->version = 1;
     rsrr->type = RSRR_INITIAL_REPLY;
     rsrr->flags = 0;
     rsrr->num = numvifs;
     
-    vif_list = (struct rsrr_vif *) (rsrr_send_buf + RSRR_HEADER_LEN);
+    vif_list = (struct rsrr_vif *)(rsrr_send_buf + RSRR_HEADER_LEN);
     
     /* Include the vif list. */
     for (vifi = 0, v = uvifs; vifi < numvifs; vifi++, v++) {
 	vif_list[vifi].id = vifi;
 	vif_list[vifi].status = 0;
-	if (v->uv_flags & (VIFF_DISABLED))
-	    BIT_SET(vif_list[vifi].status, RSRR_DISABLED_BIT);
+	if (v->uv_flags & VIFF_DISABLED)
+	    BIT_SET(vif_list[vifi].status,RSRR_DISABLED_BIT);
 	vif_list[vifi].threshold = v->uv_threshold;
 	vif_list[vifi].local_addr = v->uv_lcl_addr;
     }
@@ -240,8 +214,9 @@ rsrr_accept_iq()
     sendlen = RSRR_HEADER_LEN + numvifs*RSRR_VIF_LEN;
     
     /* Send it. */
-    IF_DEBUG(DEBUG_RSRR)
-	pimd_log(LOG_DEBUG, 0, "Send RSRR Initial Reply");
+    IF_DEBUG(DEBUG_RSRR) {
+	logit(LOG_DEBUG, 0, "Send RSRR Initial Reply");
+    }
     rsrr_send(sendlen);
 }
 
@@ -253,13 +228,9 @@ rsrr_accept_iq()
  * change notification.
  */
 /* XXX: must modify if your routing table structure/search is different */
-static int
-rsrr_accept_rq(route_query, flags, gt_notify)
-    struct rsrr_rq *route_query;
-    u_int8 flags;
-    struct gtable *gt_notify;
+static int rsrr_accept_rq(struct rsrr_rq *route_query, u_int8 flags, struct gtable *gt_notify)
 {
-    struct rsrr_header *rsrr;
+    struct rsrr_header *rsrr = (struct rsrr_header *)rsrr_send_buf;
     struct rsrr_rr *route_reply;
     int sendlen;
     struct gtable *gt;
@@ -276,7 +247,6 @@ rsrr_accept_rq(route_query, flags, gt_notify)
 #endif /* PIM */
     
     /* Set up message */
-    rsrr = (struct rsrr_header *) rsrr_send_buf;
     rsrr->version = 1;
     rsrr->type = RSRR_ROUTE_REPLY;
     rsrr->flags = flags;
@@ -381,10 +351,9 @@ rsrr_accept_rq(route_query, flags, gt_notify)
 	    route_reply->out_vif_bm = gt_notify->gt_grpmems;
 	else
 	    route_reply->out_vif_bm = 0;
-	
     } else if (find_src_grp(route_query->source_addr, 0,
 			    route_query->dest_addr)) {
-	
+
 	/* Found kernel entry. Code taken from add_table_entry() */
 	gt = gtp ? gtp->gt_gnext : kernel_table;
 	
@@ -401,7 +370,6 @@ rsrr_accept_rq(route_query, flags, gt_notify)
 	    BIT_SET(rsrr->flags, RSRR_NOTIFICATION_BIT);
 	    rsrr_cache(gt, route_query);
 	}
-	
     } else {
 	/* No kernel entry; use routing table. */
 	r = determine_route(route_query->source_addr);
@@ -429,7 +397,6 @@ rsrr_accept_rq(route_query, flags, gt_notify)
 	    /* Include the routing entry. */
 	    route_reply->in_vif = gt->gt_route->rt_parent;
 	    route_reply->out_vif_bm = gt->gt_grpmems;
-	    
 	} else {
 	    /* Set error bit. */
 	    BIT_SET(rsrr->flags, RSRR_ERROR_BIT);
@@ -437,35 +404,37 @@ rsrr_accept_rq(route_query, flags, gt_notify)
     }
 #endif /* pimd - mrouted specific code */
     
-    IF_DEBUG(DEBUG_RSRR)
-	pimd_log(LOG_DEBUG, 0, "%sSend RSRR Route Reply for src %s dst %s in vif %d out vifs 0x%x\n",
-	    gt_notify ? "Route Change: " : "",
-	    inet_fmt(route_reply->source_addr,s1),
-	    inet_fmt(route_reply->dest_addr,s2),
-	    route_reply->in_vif, route_reply->out_vif_bm);
-    
+    IF_DEBUG(DEBUG_RSRR) {
+	logit(LOG_DEBUG, 0, "%sSend RSRR Route Reply for src %s dst %s in vif %d out vif %d\n",
+	      gt_notify ? "Route Change: " : "",
+	      inet_fmt(route_reply->source_addr, s1, sizeof(s1)),
+	      inet_fmt(route_reply->dest_addr, s2, sizeof(s2)),
+	      route_reply->in_vif,route_reply->out_vif_bm);
+    }
+
     /* Send it. */
     return rsrr_send(sendlen);
 }
 
 /* Send an RSRR message. */
-static int
-rsrr_send(sendlen)
-    int sendlen;
+static int rsrr_send(int sendlen)
 {
     int error;
     
-    /* Send it. */
-    error = sendto(rsrr_socket, rsrr_send_buf, sendlen, 0,
-		   (struct sockaddr *)&client_addr, client_length);
-    
-    /* Check for errors. */
-    if (error < 0) {
-	pimd_log(LOG_WARNING, errno, "Failed send on RSRR socket");
-    } else if (error != sendlen) {
-	pimd_log(LOG_WARNING, 0,
-	    "Sent only %d out of %d bytes on RSRR socket\n", error, sendlen);
-    }
+    while ((error = sendto(rsrr_socket, rsrr_send_buf, sendlen, 0,
+			   (struct sockaddr *)&client_addr, client_length)) < 0)
+    {
+	if (errno == EINTR)
+	    continue;		/* Received signal, retry syscall. */
+
+	logit(LOG_WARNING, errno, "Failed sendto() in rsrr_send()");
+
+	return error;
+    } 
+
+    if (error != sendlen)
+	logit(LOG_WARNING, 0, "Sent only %d out of %d bytes on RSRR socket", error, sendlen);
+
     return error;
 }
 
@@ -473,15 +442,10 @@ rsrr_send(sendlen)
 /* Cache a message being sent to a client.  Currently only used for
  * caching Route Reply messages for route change notification.
  */
-static void
-rsrr_cache(gt, route_query)
-    struct gtable *gt;
-    struct rsrr_rq *route_query;
+static void rsrr_cache(struct gtable *gt, struct rsrr_rq *route_query)
 {
     struct rsrr_cache *rc, **rcnp;
-    struct rsrr_header *rsrr;
-
-    rsrr = (struct rsrr_header *) rsrr_send_buf;
+    struct rsrr_header *rsrr = (struct rsrr_header *)rsrr_send_buf;
 
 #ifdef PIM
     rcnp = &gt->rsrr_cache;
@@ -493,7 +457,7 @@ rsrr_cache(gt, route_query)
 	     route_query->source_addr) &&
 	    (rc->route_query.dest_addr == 
 	     route_query->dest_addr) &&
-	    (!strcmp(rc->client_addr.sun_path, client_addr.sun_path))) {
+	    (!strcmp(rc->client_addr.sun_path,client_addr.sun_path))) {
 	    /* Cache entry already exists.
 	     * Check if route notification bit has been cleared.
 	     */
@@ -505,26 +469,28 @@ rsrr_cache(gt, route_query)
 		/* Update */
 		/* TODO: XXX: No need to update iif, oifs, flags */
 		rc->route_query.query_id = route_query->query_id;
-		IF_DEBUG(DEBUG_RSRR)
-		    pimd_log(LOG_DEBUG, 0,
-			"Update cached query id %ld from client %s\n",
-			rc->route_query.query_id, rc->client_addr.sun_path);
+		IF_DEBUG(DEBUG_RSRR) {
+		    logit(LOG_DEBUG, 0,
+			  "Update cached query id %ld from client %s\n",
+			  rc->route_query.query_id, rc->client_addr.sun_path);
+		}
 	    }
 	    return;
 	}
 	rcnp = &rc->next;
     }
-    
+
     /* Cache entry doesn't already exist.  Create one and insert at
      * front of list.
      */
-    rc = (struct rsrr_cache *) malloc(sizeof(struct rsrr_cache));
+    rc = (struct rsrr_cache *)malloc(sizeof(struct rsrr_cache));
     if (rc == NULL)
-	pimd_log(LOG_ERR, 0, "ran out of memory");
+	logit(LOG_ERR, 0, "Ran out of memory in rsrr_cache()");
+
     rc->route_query.source_addr = route_query->source_addr;
     rc->route_query.dest_addr = route_query->dest_addr;
     rc->route_query.query_id = route_query->query_id;
-    strcpy(rc->client_addr.sun_path, client_addr.sun_path);
+    strlcpy(rc->client_addr.sun_path, client_addr.sun_path, sizeof(rc->client_addr.sun_path));
     rc->client_length = client_length;
 #ifdef PIM
     rc->next = gt->rsrr_cache;
@@ -533,25 +499,24 @@ rsrr_cache(gt, route_query)
     rc->next = gt->gt_rsrr_cache;
     gt->gt_rsrr_cache = rc;
 #endif /* PIM */
-    IF_DEBUG(DEBUG_RSRR)
-	pimd_log(LOG_DEBUG, 0, "Cached query id %ld from client %s\n",
-	    rc->route_query.query_id, rc->client_addr.sun_path);
+    IF_DEBUG(DEBUG_RSRR) {
+	logit(LOG_DEBUG, 0, "Cached query id %ld from client %s\n",
+	      rc->route_query.query_id, rc->client_addr.sun_path);
+    }
 }
 
 /* Send all the messages in the cache for particular routing entry.
  * Currently this is used to send all the cached Route Reply messages
  * for route change notification.
  */
-void
-rsrr_cache_send(gt, notify)
-    struct gtable *gt;
-    int notify;
+void rsrr_cache_send(struct gtable *gt, int notify)
 {
     struct rsrr_cache *rc, **rcnp;
     u_int8 flags = 0;
 
-    if (notify)
+    if (notify) {
 	BIT_SET(flags, RSRR_NOTIFICATION_BIT);
+    }
 
 #ifdef PIM
     rcnp = &gt->rsrr_cache;
@@ -561,10 +526,10 @@ rsrr_cache_send(gt, notify)
 
     while ((rc = *rcnp) != NULL) {
 	if (rsrr_accept_rq(&rc->route_query, flags, gt) < 0) {
-	    IF_DEBUG(DEBUG_RSRR)
-		pimd_log(LOG_DEBUG, 0,
-		    "Deleting cached query id %ld from client %s\n",
-		    rc->route_query.query_id, rc->client_addr.sun_path);
+	    IF_DEBUG(DEBUG_RSRR) {
+		logit(LOG_DEBUG, 0, "Deleting cached query id %ld from client %s\n",
+		      rc->route_query.query_id,rc->client_addr.sun_path);
+	    }
 	    /* Delete cache entry. */
 	    *rcnp = rc->next;
 	    free(rc);
@@ -578,12 +543,10 @@ rsrr_cache_send(gt, notify)
  * matching entry from (*,*,RP) or (*,G). The (*,G) entry brings up
  * any matching entries from (*,*,RP)
  */
-void
-rsrr_cache_bring_up(gt)
-    struct gtable *gt;
+void rsrr_cache_bring_up(struct gtable *gt)
 {
     struct gtable *gt_rp, *gt_wide;
-    u_int8 flags;
+    u_int8 flags = 0;
     struct rsrr_cache *rc, **rcnp;
 
     if (gt == (struct gtable *)NULL)
@@ -630,11 +593,12 @@ rsrr_cache_bring_up(gt)
 		if (rc->route_query.dest_addr == gt->group->group) {
 		    *rcnp = rc->next;
 		    if (rsrr_accept_rq(&rc->route_query, flags, gt) < 0) {
-			IF_DEBUG(DEBUG_RSRR)
-			    pimd_log(LOG_DEBUG, 0,
-				"Deleting cached query id %ld from client %s\n",
-				rc->route_query.query_id,
-				rc->client_addr.sun_path);
+			IF_DEBUG(DEBUG_RSRR) {
+			    logit(LOG_DEBUG, 0,
+				  "Deleting cached query id %ld from client %s\n",
+				  rc->route_query.query_id,
+				  rc->client_addr.sun_path);
+			}
 		    }
 		    /* Even on success have to delete it. */
 		    *rcnp = rc->next;
@@ -678,11 +642,12 @@ rsrr_cache_bring_up(gt)
 		     * to rsvpd
 		     */
 		    if (rsrr_accept_rq(&rc->route_query, flags, gt) < 0) {
-			IF_DEBUG(DEBUG_RSRR)
-			    pimd_log(LOG_DEBUG, 0,
-				"Deleting cached query id %ld from client %s\n",
-				rc->route_query.query_id,
-				rc->client_addr.sun_path);
+			IF_DEBUG(DEBUG_RSRR) {
+			    logit(LOG_DEBUG, 0,
+				  "Deleting cached query id %ld from client %s\n",
+				  rc->route_query.query_id,
+				  rc->client_addr.sun_path);
+			}
 		    }
 		    /* Even on success have to delete it. */
 		    free(rc);
@@ -708,27 +673,25 @@ rsrr_cache_bring_up(gt)
  * route change message to the reservation daemon: just plug all entries at
  * the front of the rsrr_cache chain.
  */
-void
-rsrr_cache_clean(gt)
-    struct gtable *gt;
+void rsrr_cache_clean(struct gtable *gt)
 {
     struct rsrr_cache *rc, *rc_next, **rcnp;
     struct gtable *gt_wide;
 #ifdef PIM
-    u_int8 flags;
+    u_int8 flags = 0;
 
     IF_DEBUG(DEBUG_RSRR) {
 	if (gt->flags & MRTF_SG)
-	    pimd_log(LOG_DEBUG, 0, "cleaning cache for source %s and group %s",
-		inet_fmt(gt->source->address, s1),
-		inet_fmt(gt->group->group, s2));
+	    logit(LOG_DEBUG, 0, "cleaning cache for source %s and group %s",
+		  inet_fmt(gt->source->address, s1, sizeof(s1)),
+		  inet_fmt(gt->group->group, s2, sizeof(s2)));
 	else if (gt->flags & MRTF_WC)
-	    pimd_log(LOG_DEBUG, 0, "cleaning cache for group %s and ANY sources",
-		 inet_fmt(gt->group->group, s1));
+	    logit(LOG_DEBUG, 0, "cleaning cache for group %s and ANY sources",
+		  inet_fmt(gt->group->group, s1, sizeof(s1)));
 	else if (gt->flags & MRTF_PMBR)
-	    pimd_log(LOG_DEBUG, 0,
-		"cleaning cache for ALL groups matching to RP %s",
-		inet_fmt(gt->source->address, s1));
+	    logit(LOG_DEBUG, 0,
+		  "cleaning cache for ALL groups matching to RP %s",
+		  inet_fmt(gt->source->address, s1, sizeof(s1)));
     }
     rc = gt->rsrr_cache;
     if (rc == (struct rsrr_cache *)NULL)
@@ -775,10 +738,11 @@ rsrr_cache_clean(gt)
 	}
  	while ((rc = *rcnp) != NULL) {
 	    if (rsrr_accept_rq(&rc->route_query, flags, gt_wide) < 0) {
-		IF_DEBUG(DEBUG_RSRR)
-		    pimd_log(LOG_DEBUG, 0,
-			"Deleting cached query id %ld from client %s\n",
-			rc->route_query.query_id, rc->client_addr.sun_path);
+		IF_DEBUG(DEBUG_RSRR) {
+		    logit(LOG_DEBUG, 0,
+			  "Deleting cached query id %ld from client %s\n",
+			  rc->route_query.query_id, rc->client_addr.sun_path);
+		}
 		/* Delete cache entry. */
 		*rcnp = rc->next;
 		free(rc);
@@ -791,8 +755,8 @@ rsrr_cache_clean(gt)
     
 #else
     IF_DEBUG(DEBUG_RSRR) {
-        pimd_log(LOG_DEBUG, 0, "cleaning cache for group %s\n",
-                 inet_fmt(gt->gt_mcastgrp, s1));
+        logit(LOG_DEBUG, 0, "cleaning cache for group %s\n",
+	      inet_fmt(gt->gt_mcastgrp, s1, sizeof(s1)));
     }
     rc = gt->gt_rsrr_cache;
     while (rc) {
@@ -804,10 +768,18 @@ rsrr_cache_clean(gt)
 #endif /* PIM */
 }
 
-void
-rsrr_clean()
+void rsrr_clean(void)
 {
     unlink(RSRR_SERV_PATH);
 }
 
 #endif /* RSRR */
+
+/**
+ * Local Variables:
+ *  version-control: t
+ *  indent-tabs-mode: t
+ *  c-file-style: "ellemtel"
+ *  c-basic-offset: 4
+ * End:
+ */
