@@ -27,9 +27,6 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-/*
- *  $Id: vif.c,v 1.22 2001/09/10 20:31:37 pavlin Exp $
- */
 /*
  * Part of this program has been derived from mrouted.
  * The mrouted program is covered by the license in the accompanying file
@@ -65,15 +62,14 @@ int             total_interfaces; /* Number of all interfaces: including the
 /*
  * Forward declarations.
  */
-static void start_vif      __P((vifi_t vifi));
-static void stop_vif       __P((vifi_t vifi));
-static void start_all_vifs __P(());
-static int init_reg_vif    __P(());
-static int update_reg_vif  __P((vifi_t register_vifi));
+static void start_vif      (vifi_t vifi);
+static void stop_vif       (vifi_t vifi);
+static void start_all_vifs (void);
+static int init_reg_vif    (void);
+static int update_reg_vif  (vifi_t register_vifi);
 
 
-void
-init_vifs()
+void init_vifs(void)
 {
     vifi_t vifi;
     struct uvif *v;
@@ -93,7 +89,7 @@ init_vifs()
     udp_socket = igmp_socket;
 #else
     if ((udp_socket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
-	pimd_log(LOG_ERR, errno, "UDP socket");
+	logit(LOG_ERR, errno, "UDP socket");
 #endif
 
     /*
@@ -103,14 +99,14 @@ init_vifs()
 	zero_vif(v, FALSE);
     }
 
-    pimd_log(LOG_INFO, 0, "Getting vifs from kernel");
+    logit(LOG_INFO, 0, "Getting vifs from kernel");
     config_vifs_from_kernel();
     if (disable_all_by_default) {
-       pimd_log(LOG_INFO, 0, "Disabling all vifs from kernel");
+       logit(LOG_INFO, 0, "Disabling all vifs from kernel");
        for (vifi = 0, v = uvifs; vifi < numvifs; ++vifi, ++v)
           v->uv_flags |= VIFF_DISABLED;
     }
-    pimd_log(LOG_INFO, 0, "Getting vifs from %s", configfilename);
+    logit(LOG_INFO, 0, "Getting vifs from %s", configfilename);
     config_vifs_from_file();
 
     /*
@@ -136,7 +132,7 @@ init_vifs()
     }
 
     if (enabled_vifs < 1) /* XXX: TODO: */
-	pimd_log(LOG_ERR, 0, "can't forward: %s",
+	logit(LOG_ERR, 0, "can't forward: %s",
 	    enabled_vifs == 0 ? "no enabled vifs" : "only one enabled vif");
     
     k_init_pim(igmp_socket);	/* Call to kernel to initialize structures */
@@ -170,7 +166,7 @@ zero_vif(v, t)
     v->uv_subnet	= INADDR_ANY_N;
     v->uv_subnetmask	= INADDR_ANY_N;
     v->uv_subnetbcast	= INADDR_ANY_N;
-    strncpy(v->uv_name, "", IFNAMSIZ);
+    strlcpy(v->uv_name, "", IFNAMSIZ);
     v->uv_groups	= (struct listaddr *)NULL;
     v->uv_dvmrp_neighbors = (struct listaddr *)NULL;
     NBRM_CLRALL(v->uv_nbrmap);
@@ -187,17 +183,16 @@ zero_vif(v, t)
     v->uv_pim_neighbors	= (struct pim_nbr_entry *)NULL;
     v->uv_local_pref	= default_source_preference;
     v->uv_local_metric	= default_source_metric;
-#ifdef Linux
+#ifdef __linux__
     v->uv_ifindex	= -1;
-#endif /* Linux */
+#endif /* __linux__ */
 }
 
 
 /*
  * Add a (the) register vif to the vif table.
  */
-static int
-init_reg_vif()
+static int init_reg_vif(void)
 {
     struct uvif *v;
     vifi_t i;
@@ -206,7 +201,7 @@ init_reg_vif()
     v->uv_flags = 0; /* initialization */
     if ((numvifs + 1) == MAXVIFS) {
         /* Exit the program! The PIM router must have a Register vif */
-	pimd_log(LOG_ERR, 0,
+	logit(LOG_ERR, 0,
 	    "cannot install the Register vif: too many interfaces");
 	/* To make lint happy */
 	return (FALSE);
@@ -223,7 +218,7 @@ init_reg_vif()
 #ifdef PIM_EXPERIMENTAL
     v->uv_flags |= VIFF_REGISTER_KERNEL_ENCAP;
 #endif
-    strncpy(v->uv_name,"register_vif0", IFNAMSIZ);
+    strlcpy(v->uv_name,"register_vif0", IFNAMSIZ);
     
     /* Use the address of the first available physical interface to
      * create the register vif.
@@ -235,7 +230,7 @@ init_reg_vif()
 	else
 	    break;
     if (i >= numvifs) {
-	pimd_log(LOG_ERR, 0, "No physical interface enabled");
+	logit(LOG_ERR, 0, "No physical interface enabled");
 	return -1;
     }
     v->uv_lcl_addr = uvifs[i].uv_lcl_addr;
@@ -247,8 +242,7 @@ init_reg_vif()
 }
 
 
-static void
-start_all_vifs()
+static void start_all_vifs(void)
 {
     vifi_t vifi;
     struct uvif *v;
@@ -266,11 +260,11 @@ start_all_vifs()
 	    /* Start vif if not DISABLED or DOWN */
 	    if (v->uv_flags & (VIFF_DISABLED | VIFF_DOWN)) {
 		if (v->uv_flags & VIFF_DISABLED)
-		    pimd_log(LOG_INFO, 0,
+		    logit(LOG_INFO, 0,
 			"%s is DISABLED; vif #%u out of service", 
 			v->uv_name, vifi);
 		else
-		    pimd_log(LOG_INFO, 0,
+		    logit(LOG_INFO, 0,
 			"%s is DOWN; vif #%u out of service", 
 			v->uv_name, vifi);
 	    }
@@ -287,8 +281,7 @@ start_all_vifs()
 /*
  * stop all vifs
  */
-void
-stop_all_vifs()
+void stop_all_vifs(void)
 {
     vifi_t vifi;
 
@@ -303,9 +296,7 @@ stop_all_vifs()
  * physical, register or tunnel (tunnels will be used in the future
  * when this code becomes PIM multicast boarder router.
  */
-static void 
-start_vif(vifi)
-    vifi_t vifi;
+static void start_vif(vifi_t vifi)
 {
     struct uvif *v;
     u_int32    src;
@@ -326,7 +317,7 @@ start_vif(vifi)
     
     /* Tell kernel to add, i.e. start this vif */
     k_add_vif(igmp_socket, vifi, &uvifs[vifi]);   
-    pimd_log(LOG_INFO, 0, "%s comes up; vif #%u now in service", v->uv_name, vifi);
+    logit(LOG_INFO, 0, "%s comes up; vif #%u now in service", v->uv_name, vifi);
     
     if (!(v->uv_flags & VIFF_REGISTER)) {
 	/*
@@ -354,21 +345,21 @@ start_vif(vifi)
 	 */
 	send_pim_hello(v, PIM_TIMER_HELLO_HOLDTIME);
     }
-#ifdef Linux
+#ifdef __linux__
     else {
 	struct ifreq ifr;
 	
 	memset(&ifr, 0, sizeof(struct ifreq));
-	/* strncpy(ifr.ifr_name,v->uv_name, IFNAMSIZ); */
-	strncpy(ifr.ifr_name, "pimreg", IFNAMSIZ);
+	/* strlcpy(ifr.ifr_name,v->uv_name, IFNAMSIZ); */
+	strlcpy(ifr.ifr_name, "pimreg", IFNAMSIZ);
 	if (ioctl(udp_socket, SIOGIFINDEX, (char *) &ifr) < 0) {
-	    pimd_log(LOG_ERR, errno, "ioctl SIOGIFINDEX for %s", ifr.ifr_name);
+	    logit(LOG_ERR, errno, "ioctl SIOGIFINDEX for %s", ifr.ifr_name);
 	    /* Not reached */
 	    return;
 	}
 	v->uv_ifindex = ifr.ifr_ifindex;
     }
-#endif /* Linux */
+#endif /* __linux__ */
 }
 
 
@@ -376,9 +367,7 @@ start_vif(vifi)
  * Stop a vif (either physical interface, tunnel or
  * register.) If we are running only PIM we don't have tunnels.
  */
-static void 
-stop_vif(vifi)
-    vifi_t vifi;
+static void stop_vif(vifi_t vifi)
 {
     struct uvif *v;
     struct listaddr *a;
@@ -437,7 +426,7 @@ stop_vif(vifi)
     }
 
     vifs_down = TRUE;
-    pimd_log(LOG_INFO, 0,
+    logit(LOG_INFO, 0,
 	"%s goes down; vif #%u out of service", v->uv_name, vifi);
 }		
 
@@ -449,9 +438,7 @@ stop_vif(vifi)
  * to be updated. As a result the Register vif has a new uv_lcl_addr and
  * is UP (virtually :))
  */
-static int
-update_reg_vif(register_vifi)
-vifi_t register_vifi;
+static int update_reg_vif(vifi_t register_vifi)
 {
     register struct uvif *v;
     register vifi_t vifi;
@@ -466,12 +453,12 @@ vifi_t register_vifi;
 	uvifs[register_vifi].uv_lcl_addr = uvifs[vifi].uv_lcl_addr;
 	start_vif(register_vifi);
 	IF_DEBUG(DEBUG_PIM_REGISTER | DEBUG_IF)
-	    pimd_log(LOG_NOTICE, 0, "%s has come up; vif #%u now in service",
+	    logit(LOG_NOTICE, 0, "%s has come up; vif #%u now in service",
 		uvifs[register_vifi].uv_name, register_vifi);
 	return 0;
     }
     vifs_down = TRUE;
-    pimd_log(LOG_WARNING, 0, "Cannot start Register vif: %s",
+    logit(LOG_WARNING, 0, "Cannot start Register vif: %s",
 	uvifs[vifi].uv_name);
     return(-1);
 }
@@ -483,8 +470,7 @@ vifi_t register_vifi;
  * tunnel end-points.  Ignore interfaces that have been administratively
  * disabled.
  */
-void
-check_vif_state()
+void check_vif_state(void)
 {
     register vifi_t vifi;
     register struct uvif *v;
@@ -508,18 +494,18 @@ check_vif_state()
 	if (v->uv_flags & (VIFF_DISABLED | VIFF_REGISTER))
 	    continue;
 	
-	strncpy(ifr.ifr_name, v->uv_name, IFNAMSIZ);
+	strlcpy(ifr.ifr_name, v->uv_name, IFNAMSIZ);
 	/* get the interface flags */
 	if (ioctl(udp_socket, SIOCGIFFLAGS, (char *)&ifr) < 0) {
            if (errno == ENODEV) {
-              pimd_log(LOG_NOTICE, 0, "%s has gone; vif #%u taken out of service",
+              logit(LOG_NOTICE, 0, "%s has gone; vif #%u taken out of service",
                        v->uv_name, vifi);
               stop_vif(vifi);
               vifs_down = TRUE;
               continue;
            }
            else {
-              pimd_log(LOG_ERR, errno,
+              logit(LOG_ERR, errno,
                        "check_vif_state: ioctl SIOCGIFFLAGS for %s", ifr.ifr_name);
            }
 	}
@@ -532,7 +518,7 @@ check_vif_state()
 	}
 	else {
 	    if (!(ifr.ifr_flags & IFF_UP)) {
-		pimd_log(LOG_NOTICE, 0,
+		logit(LOG_NOTICE, 0,
 		    "%s has gone down; vif #%u taken out of service",
 		    v->uv_name, vifi);
 		stop_vif(vifi);
@@ -581,9 +567,7 @@ check_vif_state()
  * Local addresses are excluded.
  * Return the vif number or NO_VIF if not found.
  */
-vifi_t 
-find_vif_direct(src)
-    u_int32 src;
+vifi_t find_vif_direct(u_int32 src)
 {
     vifi_t vifi;
     register struct uvif *v;
@@ -620,9 +604,7 @@ find_vif_direct(src)
  * Checks if src is local address. If "yes" return the vif index,
  * otherwise return value is NO_VIF.
  */
-vifi_t
-local_address(src)
-    u_int32 src;
+vifi_t local_address(u_int32 src)
 {
     vifi_t vifi;
     register struct uvif *v;
@@ -647,9 +629,7 @@ local_address(src)
  * (Register and tunnels excluded).
  * Return the vif number or NO_VIF if not found.
  */
-vifi_t 
-find_vif_direct_local(src)
-    u_int32 src;
+vifi_t find_vif_direct_local(u_int32 src)
 {
     vifi_t vifi;
     register struct uvif *v;
@@ -686,8 +666,7 @@ find_vif_direct_local(src)
  * Returns the highest address of local vif that is UP and ENABLED.
  * The VIFF_REGISTER interface(s) is/are excluded.
  */
-u_int32
-max_local_address()
+u_int32 max_local_address(void)
 {
     vifi_t vifi;
     struct uvif *v;
@@ -703,3 +682,12 @@ max_local_address()
     }
     return(max_address);
 }
+
+/**
+ * Local Variables:
+ *  version-control: t
+ *  indent-tabs-mode: t
+ *  c-file-style: "ellemtel"
+ *  c-basic-offset: 4
+ * End:
+ */
